import os

from dragonpanda import API_KEY
from .http_client import HttpClient
from .datasets import Datasets
from .automodels import Automodels
from .predictions import Predictions
from ._version import __version__


class DragonPandaClient(object):
    def __init__(self, key=None, uri='https://api.dragonpanda.net/api/v2', authorization_type=None):
        self._key = key or os.environ.get(API_KEY)
        if uri.endswith('/'):
            uri = uri[:-1]
        self._uri = uri
        self._authorization_type = authorization_type or ''
        self._client = HttpClient(self._key, self._uri, self._authorization_type)
        self._datasets = Datasets(self._client)
        self._automodels = Automodels(self._client)
        self._predictions = Predictions(self._client)

    @property
    def datasets(self):
        """Dataset based API operations"""
        return self._datasets

    @property
    def automodels(self):
        """Automodel based API operations"""
        return self._automodels

    @property
    def predictions(self):
        """Prediction based API operations"""
        return self._predictions

    def fit(self, file_path, target):
        """
        Generate a new model given a file and the column from the file you want to predict.

        Parameters
        ----------
        file_path : str
            The path to the file containg the data you wish to model.
        target : str
            The name of a column in the file you wish to predict.

        Returns
        -------
        Automodel
            The metadata about the model that has been created
        """

        ds = self.datasets.import_file(file_path).get_result()
        return self.automodels.fit(ds.dataset_id, target).get_result()

    def predict(self, automodel_id, file_path):
        """
        Run the predictions on new data for a model previously generated

        Parameters
        ----------
        automodel_id : str
            The id of the Automodel previously `fit` to a particular dataset
        file_path : str
            The path to a file containing all of the columns for the model
            except the 'target'

        Returns
        -------
        PredictionList
            A paged list of the predictions generated for the data from the input file.
            Each prediction maps a row number and the prediction value back to the
            row number from the original file.
        """

        return self.predictions.predict(automodel_id, file_path).get_result()

    def __str__(self):
        return "{} version {} connecting to '{}' with key '{}' and authorization type '{}'".format(
            self.__class__.__name__, __version__, self._uri, self._key, self._authorization_type)
