from datetime import datetime, date
import json
import requests
from ._version import __version__


def _process_response(response):
    if response is None:
        return None, 0, None

    # content type is probably something like 'application/json; charset: utf-8'
    # so this processes that to be useful
    content_type_value = response.headers.get('content-type')
    content_type = None if content_type_value is None else content_type_value.split(';')[0]

    if content_type == 'application/json':
        return response.json(), response.status_code, response.headers
    else:
        return response.content, response.status_code, response.headers


def _json_encode(obj):
    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    raise TypeError("Type %s not serializable" % type(obj))


class ClientError(Exception):
    """
    A ClientError is generated when the status code in the response from the Dragon Panda
    server is 400 or greater.
    """

    def __init__(self, url, status, body):
        super(ClientError, self).__init__()
        self._url = url
        self._status = status
        self._error_details = body

    @property
    def url(self):
        return self._url

    @property
    def status(self):
        return self._status

    @property
    def error_details(self):
        return self._error_details

    def __str__(self):
        return 'Error {} returned from {}. Details:\n{}'.format(self._status, self._url, self._error_details)


class InputError(Exception):
    """
    An InputError is generated when an error is found in the input given to the client library.
    This is almost certainly a bug in the client library code. Please file a bug report at:
    https://github.com/jeffesp/dragonpandaclient-py/issues/new
    """

    def __init__(self, message):
        super(InputError, self).__init__()
        self._message = message

    def __str__(self):
        return 'Input error: {}'.format(self._message)


class HttpClient(object):
    def __init__(self, key, uri, authorization_type=None):
        self._key = key
        self._uri = (uri[0:-1] if uri.endswith('/') else uri).strip()
        self._authorization_header_template = '{}'
        if authorization_type:
            self._authorization_header_template = '{} {}'.format(authorization_type, '{}')

    def _generate_headers(self):
        return {
            'Authorization': self._authorization_header_template.format(self._key),
            'User-Agent': 'DragonPanda-Python-API-Client/{}'.format(__version__),
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

    def _get_uri(self, fragment):
        # process the fragment to what we need
        if fragment.startswith(self._uri):
            fragment = fragment[len(self._uri):]
        if fragment.startswith('/'):
            fragment = fragment[1:]

        # now check if fragment is valid
        if fragment.startswith('http://') or fragment.startswith('https://'):
            raise InputError('Fully qualified URL {} given but was expecting it to be {}'.format(fragment, self._uri))

        return '{}/{}'.format(self._uri, fragment)

    def _process_args(self, args):
        # remove arg we don't want to pass to base requests lib, don't care about it's value anymore
        args.pop('raw_response', None)

        # if headers specified, then generate defaults and update with the ones specified
        # or just set them if none specified
        default_headers = self._generate_headers()
        if 'headers' in args:
            user_headers = args['headers']
            args['headers'] = default_headers
            args['headers'].update(user_headers)
        else:
            args['headers'] = default_headers

        # copy data to json for proper serialization
        if 'data' in args and args['headers']['Content-Type'] == 'application/json':
            args['data'] = json.dumps(args['data'], default=_json_encode)

        args['allow_redirects'] = False

        return args

    def request_with_headers(self, verb, uri_path, **kwargs):
        early_return = 'raw_response' in kwargs and kwargs['raw_response']
        response = requests.request(verb, self._get_uri(uri_path), **self._process_args(kwargs))

        if early_return:
            return response, None, None

        if response.ok:
            return _process_response(response)
        else:
            raise ClientError(uri_path, response.status_code, response.text)

    def request(self, verb, uri_path, **kwargs):
        response, _, _ = self.request_with_headers(verb, uri_path, **kwargs)
        return response
