from time import sleep
import random
import logging


logger = logging.getLogger(__name__)


class TaskError(Exception):
    """
    Raised during execution of Task.get_result if the task errored or was aborted
    """


class TaskTimeout(Exception):
    """
    Raised if a task times out before it completes.
    """


class Task(object):
    """
    This object is used to wait for asynchronous tasks to complete.

    Construction of the object requires an instance of `HttpClient` used to make authenticated
    API calls, the URL that was given back as the status checking URL, and a function to run when
    the task is complete. The function will receive the URL returned from the status endpoint as
    the location of the new object. The function should return a value that will be used as the
    result of the Task.

    Calling `get_result()` on this object will block until the operation completes, times out, or
    fails. It will return the result of the asynchronous operation once it is complete. If it
    times out or fails the return value is `None` and the `is_complete`, `timed_out` and
    `is_successful` properties should be used to determine the state of the result.

    It has two boolean properties (`is_complete` and `successful`) that can be used to implement
    your own polling algorithm. Once `is_complete` is `True`, `get_result()` will return the
    result if `successful` or `None` if not `successful`.
    """

    default_timeout = 30 * 60  # 30 minutes
    max_wait = 5 * 60  # 5 minutes

    def __init__(self, base_client, task_url, get_callback):
        self.client = base_client
        self.task_url = task_url
        self.get_callback = get_callback
        self.complete = False
        self.successful = False
        self.did_time_out = False
        self.result = None
        self.error = None
        self.status = None
        self.message = None

    def get_result(self, timeout=None):
        if timeout == 0 and not self.complete:
            self._check_complete()
        else:
            current_timeout = timeout or self.default_timeout
            elapsed = 0
            i = 0
            while True:
                # if we timed out, say so and exit the loop
                if elapsed >= current_timeout:
                    self.did_time_out = True
                    raise TaskTimeout

                # AWS says adding jitter to your exponential backoff is good:
                # https://aws.amazon.com/blogs/architecture/exponential-backoff-and-jitter/
                # current configuration makes it exponential until about 10 iterations,
                # and then uses `max_wait`
                wait = random.uniform(0, min(self.max_wait, 2**i))
                logger.debug('Waiting {} seconds'.format(wait))
                sleep(wait)

                # after sleeping, check status and quit if done
                if self._check_complete():
                    break

                # update loop variables
                i = i + 1
                elapsed = elapsed + wait

        if self.result is not None:
            return self.result
        raise TaskError('Task did not complete successfully: {}'.format(self.error))

    def is_complete(self):
        """`True` when the operation has completed."""
        return self._check_complete()

    @property
    def is_successful(self):
        """
        `True` when the operation was successful, `False` when the API indicated there was an
        error with the operation.
        """
        return self.successful

    @property
    def timed_out(self):
        """
        `True` if a timeout to `get_result()` was given and the operation took longer than that,
        or if the `default_timeout` was execeeded.
        """
        return self.did_time_out

    def _check_complete(self):
        if self.complete:
            return True

        response, status, headers = self.client.request_with_headers(
            'GET', self.task_url)
        if status == 303:
            self.complete = True
            self.successful = True
            self.status = 'COMPLETE'
            self.message = ''
            self.result = self.get_callback(headers['location'])
            return True
        else:
            self.status = response['status']
            self.message = response['message']

            if response['status'].lower()[0:5] in ('error', 'abort'):
                self.complete = True
                self.successful = False
                self.error = response
                return True

            log_msg = '{status}: {detail}'.format(
                status=response['status'],
                detail=response['message']
            )
            logger.info(log_msg)

            return False
