import json
import pytest

from dragonpanda.http_client import HttpClient, ClientError
from dragonpanda.automodels import Automodels

sample_automodel = {
    'automodelId': 'tango',
    'datasetId': 'sierra',
    'name': 'uniform',
    'target': 'romeo',
    'created': 'NOW',
    'recommendedModels': [{
        'id': 'victor',
        'modelBadge': ['whiskey', 'xray'],
        'modelType': 'yankee',
        'metrics': {
            'zulu': 0.1
        }
    }]
}

sample_features = {
    'automodelId': 'tango',
    'target': 'credit_risk',
    'datasetId': 'sierra',
    'features': [{
        'type': 'Numeric',
        'name': 'Credit_Amount'
    }, {
        'type': 'Categorical',
        'name': 'credit_risk'
    }]
}

sample_automodel_list = {'count': 2, 'data': [sample_automodel, sample_automodel]}


def test_automodel_get(requests_mock):
    requests_mock.register_uri(
        'GET',
        'http://example.com/automodels/tango/',
        headers={'content-type': 'application/json'},
        json=sample_automodel)
    client = HttpClient('a-key', 'http://example.com')
    ams = Automodels(client)

    result = ams.get('tango')

    assert result.automodel_id == 'tango'
    assert result.dataset_id == 'sierra'
    assert result.name == 'uniform'
    assert result.target == 'romeo'
    assert result.created_on == 'NOW'
    assert len(result.models) == 1
    assert result.models[0].badges == ['whiskey', 'xray']
    assert result.models[0].model_type == 'yankee'
    assert result.models[0].metrics['zulu'] == 0.1


def test_automodel_get_features(requests_mock):
    requests_mock.register_uri(
        'GET',
        'http://example.com/automodels/tango/features/',
        headers={'content-type': 'application/json'},
        json=sample_features)
    client = HttpClient('a-key', 'http://example.com')
    ams = Automodels(client)

    result = ams.get_features('tango')

    assert result.automodel_id == 'tango'
    assert result.dataset_id == 'sierra'
    assert result.target == 'credit_risk'
    assert len(result.feature_list) == 2
    assert result.feature_list[0].name == 'Credit_Amount'
    assert result.feature_list[0].feature_type == 'Numeric'
    assert result.feature_list[1].name == 'credit_risk'
    assert result.feature_list[1].feature_type == 'Categorical'


def test_automodel_get_not_found_raises_error(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/automodels/abc123/', status_code=404)

    client = HttpClient('a-key', 'http://example.com')
    ams = Automodels(client)

    with pytest.raises(ClientError) as error:
        ams.get('abc123')

    assert error.value.status is 404


def test_automodel_fit(requests_mock):
    def match_request_body(request):
        return json.loads(request.text) == {'datasetId': 'a-dataset', 'target': 'a-target'}

    requests_mock.register_uri(
        'POST',
        'http://example.com/api/automodels/',
        status_code=202,
        headers={'location': 'http://example.com/api/status/foobar'},
        additional_matcher=match_request_body)

    client = HttpClient('a-key', 'http://example.com/api')
    ams = Automodels(client)
    result = ams.fit('a-dataset', 'a-target')

    assert result.task_url == 'http://example.com/api/status/foobar'


def test_automodel_fit_error(requests_mock):
    requests_mock.register_uri('POST', 'http://example.com/api/automodels/', status_code=403)

    client = HttpClient('a-key', 'http://example.com/api')
    ams = Automodels(client)

    with pytest.raises(ClientError) as error:
        ams.fit('a-dataset', 'a-target')

    assert error.value.url == 'automodels/'
    assert error.value.status == 403


def test_automodel_list(requests_mock):
    requests_mock.register_uri(
        'GET',
        'http://example.com/api/automodels/?offset=0&limit=50',
        headers={'content-type': 'application/json'},
        json=sample_automodel_list)

    client = HttpClient('a-key', 'http://example.com/api')
    ams = Automodels(client)

    result = ams.list()

    assert result.count == 2
    assert len(result.automodels) == 2
