import json
from dragonpanda.client import DragonPandaClient


def test_client_creation():
    client = DragonPandaClient()
    assert client is not None


def test_client_has_default_values():
    client = DragonPandaClient()
    assert client._key is None
    assert client._uri == 'https://api.dragonpanda.net/api/v2'
    assert client._authorization_type == ''


def test_client_accepts_key():
    client = DragonPandaClient('alpha-beta')
    assert client._key == 'alpha-beta'


def test_client_accepts_uri():
    client = DragonPandaClient(uri='https://api.staging.dragonpanda.net')
    assert client._uri == 'https://api.staging.dragonpanda.net'


def test_client_accepts_authorization_type():
    client = DragonPandaClient(authorization_type='Token')
    assert client._authorization_type == 'Token'


def test_client_str_is_good():
    client = DragonPandaClient(key='foobar', uri='some://url', authorization_type='Token')
    assert str(client) == "DragonPandaClient version 1.0.0 connecting to 'some://url' with key 'foobar' " \
                          "and authorization type 'Token'"


def test_client_fit_all_in_one(requests_mock):
    # initial file import
    requests_mock.register_uri(
        'POST',
        'http://example.com/api/datasets/import/file/',
        status_code=202,
        headers={'location': 'http://example.com/api/status/alpha-beta'})
    requests_mock.get('http://example.com/api/status/alpha-beta', [{
        'status_code': 303,
        'text': 'something here',
        'headers': {
            'location': 'http://example.com/api/datasets/charlie-delta'
        }
    }])
    # get after upload resolution
    requests_mock.register_uri(
        'GET',
        'http://example.com/api/datasets/charlie-delta',
        headers={'content-type': 'application/json'},
        json={
            'id': 'charlie-delta',
            'name': 'some-file.csv',
            'created': 'sometime'
        })

    # fit on uploaded dataset request
    def match_request_body(request):
        return json.loads(request.text) == {'datasetId': 'charlie-delta', 'target': 'a-target'}

    requests_mock.register_uri(
        'POST',
        'http://example.com/api/automodels/',
        status_code=202,
        headers={'location': 'http://example.com/api/status/echo-foxtrot'},
        additional_matcher=match_request_body)
    requests_mock.get('http://example.com/api/status/echo-foxtrot', [{
        'status_code': 303,
        'text': 'something here',
        'headers': {
            'location': 'http://example.com/api/automodels/golf-hotel'
        }
    }])
    # get after modeling complete
    requests_mock.register_uri(
        'GET',
        'http://example.com/api/automodels/golf-hotel/',
        headers={'content-type': 'application/json'},
        json={
            'automodelId': 'golf-hotel',
            'datasetId': 'charlie-delta',
            'name': 'india-juliet',
            'target': 'a-target',
            'created': 'NOW',
            'recommendedModels': [{
                'id': 'kilo-lima',
                'modelBadge': ['mike', 'november'],
                'modelType': 'oscar',
                'metrics': {
                    'papa': 0.1
                }
            }]
        })

    client = DragonPandaClient(key='foobar', uri='http://example.com/api')
    result = client.fit('tests/some-file.csv', 'a-target')

    assert result.automodel_id == 'golf-hotel'


def test_client_predict(requests_mock):
    # predict request and status request
    requests_mock.register_uri(
        'POST',
        'http://example.com/api/automodels/hotel-india/predictions/',
        status_code=202,
        headers={'location': 'http://example.com/api/status/november-oscar'})
    requests_mock.get('http://example.com/api/status/november-oscar', [{
        'status_code': 303,
        'text': 'something here',
        'headers': {
            'location': 'http://example.com/api/automodels/hotel-inda/predictions/papa-quebec'
        }
    }])
    # get after prediction resolution
    requests_mock.register_uri(
        'GET',
        'http://example.com/api/automodels/hotel-india/predictions/papa-quebec/?offset=0&limit=50',
        headers={'content-type': 'application/json'},
        json={
            'id': 'papa-quebec',
            'count': 1,
            'data': [{
                'rowId': 0,
                'prediction': 123,
                'predictionValues': []
            }]
        })

    client = DragonPandaClient(key='foobar', uri='http://example.com/api')
    result = client.predict('hotel-india', 'tests/some-file.csv')

    assert result.prediction_id == 'papa-quebec'
