import pytest

from dragonpanda.http_client import HttpClient, ClientError
from dragonpanda.datasets import Datasets


def test_dataset_list(requests_mock):
    requests_mock.register_uri(
        'GET',
        'http://example.com/datasets/?offset=0&limit=50',
        headers={'content-type': 'application/json'},
        json={
            'data': [{
                'id': 'echo-foxtrot',
                'name': 'ef.csv',
                'created': 'sometime'
            }],
            'count': 1
        })

    client = HttpClient('a-key', 'http://example.com')
    ds = Datasets(client)

    result = ds.list()

    assert result.count == 1
    assert len(result.items) == 1

def test_dataset_list_with_params(requests_mock):
    requests_mock.register_uri(
        'GET',
        'http://example.com/datasets/?offset=1&limit=10',
        headers={'content-type': 'application/json'},
        json={
            'data': [{
                'id': 'echo-foxtrot',
                'name': 'ef.csv',
                'created': 'sometime'
            }],
            'count': 1
        })

    client = HttpClient('a-key', 'http://example.com')
    ds = Datasets(client)

    result = ds.list(offset=1, limit=10)

    assert result.count == 1
    assert len(result.items) == 1


def test_dataset_get(requests_mock):
    requests_mock.register_uri(
        'GET',
        'http://example.com/datasets/echo-foxtrot',
        headers={'content-type': 'application/json'},
        json={
            'id': 'echo-foxtrot',
            'name': 'ef.csv',
            'created': 'sometime'
        })

    client = HttpClient('a-key', 'http://example.com')
    ds = Datasets(client)

    result = ds.get('echo-foxtrot')

    assert result.dataset_id == 'echo-foxtrot'
    assert result.name == 'ef.csv'
    assert result.created_on == 'sometime'


def test_dataset_get_not_found_raises_error(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/datasets/abc123', status_code=404)

    client = HttpClient('a-key', 'http://example.com')
    ds = Datasets(client)

    with pytest.raises(ClientError) as error:
        ds.get('abc123')

    assert error.value.status is 404


def test_dataset_import_file(requests_mock):
    requests_mock.register_uri(
        'POST',
        'http://example.com/api/datasets/import/file/',
        status_code=202,
        headers={'location': 'http://example.com/api/status/foobar'})

    client = HttpClient('a-key', 'http://example.com/api')
    ds = Datasets(client)
    result = ds.import_file('tests/some-file.csv')

    assert result.task_url == 'http://example.com/api/status/foobar'

def test_dataset_import_url(requests_mock):
    def match_body(request):
        return request.text == '{"url": "http://example.com/echo/xray"}'
    requests_mock.register_uri(
        'POST',
        'http://example.com/api/datasets/import/url/',
        additional_matcher=match_body,
        status_code=202,
        headers={'location': 'http://example.com/api/status/foobar'})

    client = HttpClient('a-key', 'http://example.com/api')
    ds = Datasets(client)
    result = ds.import_url('http://example.com/echo/xray')

    assert result.task_url == 'http://example.com/api/status/foobar'

def test_dataset_import_error(requests_mock):
    requests_mock.register_uri(
        'POST',
        'http://example.com/api/datasets/import/file/',
        status_code=500)
        
    client = HttpClient('a-key', 'http://example.com/api')
    ds = Datasets(client)

    with pytest.raises(ClientError) as error:
        ds.import_file('tests/some-file.csv')

    assert error.value.url == 'datasets/import/file/'
    assert error.value.status == 500
