import pytest

from dragonpanda.http_client import HttpClient, ClientError, InputError


def test_adds_key_as_authorization_header(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/api/testing',
                               body='', request_headers={'Authorization': 'abc123'})
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', 'testing')
    assert response is not None


def test_prepends_authorization_type(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/api/testing',
                               body='', request_headers={'Authorization': 'Token abc123'})
    client = HttpClient('abc123', 'http://example.com/api', authorization_type='Token')
    response = client.request('GET', 'testing')
    assert response is not None


def test_appends_path_to_base_url_from_construction(requests_mock):
    requests_mock.register_uri(
        'GET', 'http://example.com/api/testing', body='')
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', 'testing')
    assert response is not None


def test_default_headers_generated(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/api/testing',
                               request_headers={
                                   'Authorization': 'abc123',
                                   'User-Agent': 'DragonPanda-Python-API-Client/1.0.0',
                                   'Content-Type': 'application/json',
                                   'Accept': 'application/json'
                               },
                               headers={'content-type': 'application/json'},
                               json={'foo': 'bar'})
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', 'testing')
    assert response == {'foo': 'bar'}


def test_custom_headers_used_if_provided(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/api/testing',
                               request_headers={'golf': 'hotel'},
                               headers={'content-type': 'application/json'},
                               json={'india': 'juliet'})
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', 'testing', headers={'golf': 'hotel'})
    assert response == {'india': 'juliet'}


def test_url_fragment_beginning_slash_removed(requests_mock):
    requests_mock.register_uri(
        'GET', 'http://example.com/api/testing', body='')
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', '/testing')
    assert response is not None


def test_raises_ClientError_on_failure(requests_mock):
    with pytest.raises(ClientError) as error:
        requests_mock.register_uri(
            'GET', 'http://example.com/api/testing', status_code=404)
        client = HttpClient('abc123', 'http://example.com/api')
        client.request('GET', 'testing')

    assert error.value.status == 404


def test_json_encodes_body_when_provided(requests_mock):
    def match_request_body(request):
        return request.body == '{"alpha": "zulu"}'
    requests_mock.register_uri('GET', 'http://example.com/api/testing',
                               additional_matcher=match_request_body, text='')
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', 'testing', data={'alpha': 'zulu'})
    assert response is not None


def test_does_not_json_encode_body_when_not_json_application_type(requests_mock):
    def match_request_body(request):
        return request.body == '<element>value</element>'
    requests_mock.register_uri('GET', 'http://example.com/api/testing',
                               additional_matcher=match_request_body, text='')
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', 'testing', data='<element>value</element>',
                              headers={'Content-Type': 'application/xml'})
    assert response is not None


def test_returned_body_parsed_as_json_when_content_type_is_json(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/api/whiskey',
                               status_code=200,
                               headers={'content-type': 'application/json'},
                               json={'yankee': 'xray'})
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', '/whiskey')
    assert response == {'yankee': 'xray'}


def test_returned_body_returned_when_content_type_not_json(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/api/whiskey',
                               status_code=200,
                               headers={'content-type': 'application/xml'},
                               text='<element>value</element>')
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', '/whiskey')
    assert response == b'<element>value</element>'


def test_normalized_url_when_given_fully_qualified(requests_mock):
    requests_mock.register_uri(
        'GET', 'http://example.com/api/testing', body='')
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', 'http://example.com/api/testing')
    assert response is not None


def test_raises_InputError_when_domain_names_do_not_match(requests_mock):
    with pytest.raises(InputError):
        requests_mock.register_uri(
            'GET', 'http://example.com/api/testing', body='')
        client = HttpClient('abc123', 'http://example.com/api')
        client.request('GET', 'http://example.org/api/testing')


def test_will_return_raw_request_when_asked_for(requests_mock):
    requests_mock.register_uri('GET', 'http://example.com/api/testing', text='stuff')
    client = HttpClient('abc123', 'http://example.com/api')
    response = client.request('GET', 'http://example.com/api/testing', raw_response=True)

    assert response.text == 'stuff'