import pytest
from dragonpanda.http_client import HttpClient, ClientError
from dragonpanda.predictions import Predictions

sample_predictions = {
    'id': 'prediction-id',
    'count': 2,
    'automodelId': 'automodel-id',
    'data': [{
        'rowId': 0,
        'prediction': 'alpha',
        'predictionValues': [{
            'value': 0.49,
            'label': 'beta'
        }, {
            'value': 0.51,
            'label': 'alpha'
        }]
    }, {
        'rowId': 1,
        'prediction': 'beta',
        'predictionValues': [{
            'value': 0.49,
            'label': 'alpha'
        }, {
            'value': 0.51,
            'label': 'beta'
        }]
    }]
}


def test_prediction_predict_returns_task(requests_mock):
    requests_mock.register_uri(
        'POST',
        'http://example.com/api/automodels/automodel-id/predictions/',
        status_code=202,
        headers={'location': 'http://example.com/api/status/foobar'})

    client = HttpClient('a-key', 'http://example.com/api')
    prediction = Predictions(client)
    result = prediction.predict('automodel-id', 'tests/some-file.csv')

    assert result.task_url == 'http://example.com/api/status/foobar'

def test_prediction_predict_error_response(requests_mock):
    requests_mock.register_uri(
        'POST',
        'http://example.com/api/automodels/automodel-id/predictions/',
        status_code=500)

    client = HttpClient('a-key', 'http://example.com/api')
    prediction = Predictions(client)

    with pytest.raises(ClientError) as error:
        prediction.predict('automodel-id', 'tests/some-file.csv')

    assert error.value.url == 'automodels/automodel-id/predictions/'
    assert error.value.status == 500


def test_prediction_predict_errors_with_bad_id(requests_mock):
    requests_mock.register_uri(
        'POST', 'http://example.com/api/automodels/automodel-id/predictions/', status_code=404)

    client = HttpClient('a-key', 'http://example.com/api')
    prediction = Predictions(client)
    with pytest.raises(ClientError) as error:
        prediction.predict('automodel-id', 'tests/some-file.csv')

    assert error.value.status is 404


def tests_get_predictions_returns_predictions(requests_mock):
    requests_mock.register_uri(
        'GET',
        'http://example.com/api/automodels/automodel-id/predictions/prediction-id/?offset=0&limit=50',
        status_code=200,
        headers={'content-type': 'application/json'},
        json=sample_predictions)

    client = HttpClient('a-key', 'http://example.com/api')
    prediction = Predictions(client)
    result = prediction.get('automodel-id', 'prediction-id')

    assert result.prediction_id == 'prediction-id'
    assert result.count == 2
    assert result.predictions[0].row == 0
    assert result.predictions[0].prediction == 'alpha'
    assert result.predictions[0].values == [{'label': 'beta', 'value': 0.49}, {'label': 'alpha', 'value': 0.51}]
    assert result.predictions[1].row == 1
    assert result.predictions[1].prediction == 'beta'
    assert result.predictions[1].values == [{'label': 'alpha', 'value': 0.49}, {'label': 'beta', 'value': 0.51}]


def test_get_predictions_with_params(requests_mock):
    requests_mock.register_uri(
        'GET',
        'http://example.com/api/automodels/automodel-id/predictions/prediction-id/?offset=3&limit=10',
        status_code=200,
        headers={'content-type': 'application/json'},
        json=sample_predictions)

    client = HttpClient('a-key', 'http://example.com/api')
    prediction = Predictions(client)
    result = prediction.get('automodel-id', 'prediction-id', 3, 10)

    assert result.prediction_id == 'prediction-id'
    assert result.count == 2
